'use strict';

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('Stream', function () {
  it('returns an object', function () {
    var result = new _index2.default(function () {});

    (0, _expect2.default)(result).toBeAn(Object);
  });

  describe('publisher', function () {
    var spy = void 0,
        each = void 0;

    beforeEach(function () {
      each = (0, _expect.createSpy)();
      spy = (0, _expect.createSpy)();
    });

    it('does not invoke the publisher immediately', function () {
      _index2.default.create(spy);

      (0, _expect2.default)(spy).toNotHaveBeenCalled();
    });

    it('invokes the publisher when observed', function () {
      var stream = new _index2.default(spy);
      stream.forEach(function () {});

      (0, _expect2.default)(spy).toHaveBeenCalled();
    });

    it('passes a push & complete method', function () {
      var stream = new _index2.default(spy);
      stream.forEach(function () {});

      var _spy$calls$0$argument = (0, _slicedToArray3.default)(spy.calls[0].arguments, 2),
          push = _spy$calls$0$argument[0],
          complete = _spy$calls$0$argument[1];

      (0, _expect2.default)(push).toBeA(Function);
      (0, _expect2.default)(complete).toBeA(Function);
    });

    it('throws if the publisher is not a function', function () {
      var fail = function fail() {
        return new _index2.default(true);
      };

      (0, _expect2.default)(fail).toThrow(/function/i);
    });

    it('sends values to all subscribers', function () {
      var msg = {};
      spy.andCall(function (push) {
        return push(msg);
      });
      var stream = new _index2.default(spy);
      stream.forEach(each);

      (0, _expect2.default)(each).toHaveBeenCalled();
      (0, _expect2.default)(each).toHaveBeenCalledWith(msg, undefined);
    });

    it('does not unnecessarily invoke the publisher', function () {
      var stream = new _index2.default(spy);
      stream.forEach(function () {});
      stream.forEach(function () {});

      (0, _expect2.default)(spy.calls.length).toBe(1);
    });

    it('disposes of the stream when all listeners are removed', function () {
      var spy = (0, _expect.createSpy)();
      var stream = new _index2.default(function () {
        return spy;
      });

      stream.forEach(function () {}).dispose();

      (0, _expect2.default)(spy).toHaveBeenCalled();
    });

    it('opens the stream again when observed', function () {
      var open = (0, _expect.createSpy)();
      var stream = new _index2.default(open);

      stream.forEach(function () {}).dispose();
      (0, _expect2.default)(open).toHaveBeenCalled();

      open.reset();

      stream.forEach(function () {}).dispose();
      (0, _expect2.default)(open).toHaveBeenCalled();
    });

    it('allows two parameters', function () {
      var stream = new _index2.default(function (push) {
        return push(1, 2, 3);
      });
      stream.forEach(spy);

      (0, _expect2.default)(spy).toHaveBeenCalledWith(1, 2);
    });
  });

  describe('complete', function () {
    it('is a promise', function () {
      var _ref = new _index2.default((0, _expect.createSpy)()),
          complete = _ref.complete;

      (0, _expect2.default)(complete).toExist();
      (0, _expect2.default)(complete.then).toBeA(Function);
      (0, _expect2.default)(complete.catch).toBeA(Function);
    });

    it('completes when asked', function () {
      var stream = new _index2.default(function (push, complete) {
        return complete();
      });

      // Force the stream open.
      stream.forEach(function () {});

      // Mocha should not time out.
      return stream.complete;
    });

    it('throws if push is called after completion', function () {
      var stream = new _index2.default(function (push, complete) {
        complete();
        push('this should throw.');
      });

      var forEach = stream.forEach.bind(stream);
      var fail = function fail() {
        return forEach(function () {});
      };

      (0, _expect2.default)(fail).toThrow(/closed/i);
    });

    it('throws if complete is called twice', function () {
      var stream = new _index2.default(function (push, complete) {
        complete();
        complete();
      });

      var fail = function fail() {
        return stream.forEach(function () {});
      };

      (0, _expect2.default)(fail).toThrow(/(complete|close)/i);
    });
  });

  describe('forEach()', function () {
    var stream = void 0,
        each = void 0,
        push = void 0;

    beforeEach(function () {
      each = (0, _expect.createSpy)();
      var publisher = (0, _expect.createSpy)();
      stream = new _index2.default(publisher);

      publisher.andCall(function (next) {
        push = next;
      });
    });

    it('subscribes to real-time events', function () {
      stream.forEach(each);
      (0, _expect2.default)(each).toNotHaveBeenCalled();

      push('hello');
      (0, _expect2.default)(each).toHaveBeenCalledWith('hello', undefined);

      push('this is stream');
      (0, _expect2.default)(each).toHaveBeenCalledWith('this is stream', undefined);
    });

    it('returns an object', function () {
      var result = stream.forEach(each);

      (0, _expect2.default)(result).toBeAn(Object);
    });

    it('unsubscribes on demand', function () {
      var result = stream.forEach(each);

      result.dispose();
      push('new event');

      (0, _expect2.default)(each).toNotHaveBeenCalled();
    });

    it('does not remove other listeners when calling dispose', function () {
      var other = (0, _expect.createSpy)();
      var result = stream.forEach(each);
      stream.forEach(other);

      result.dispose();
      push('message');

      (0, _expect2.default)(other).toHaveBeenCalled();
    });
  });

  describe('static "fromEvent"', function () {
    var emitter = void 0,
        stream = void 0;

    beforeEach(function () {
      emitter = new _events2.default();
      stream = _index2.default.fromEvent(emitter, 'message');
    });

    it('forwards events from the stream', function () {
      var spy = (0, _expect.createSpy)();
      stream.forEach(spy);
      emitter.emit('message');

      (0, _expect2.default)(spy).toHaveBeenCalled();
    });

    it('unsubscribes when there are no observers', function () {
      var subscription = stream.forEach(function () {});
      (0, _expect2.default)(emitter.listenerCount('message')).toBe(1);

      subscription.dispose();
      (0, _expect2.default)(emitter.listenerCount('message')).toBe(0);
    });

    it('works on DOM-style events', function () {
      emitter.removeEventListener = emitter.removeListener;
      emitter.addEventListener = emitter.on;
      emitter.removeListener = undefined;
      emitter.on = undefined;

      var stream = _index2.default.fromEvent(emitter, 'disconnect');

      var spy = (0, _expect.createSpy)();
      stream.forEach(spy).dispose();
    });
  });
}); /* eslint-env mocha */